declare type KeyValue = {
    [key: string]: unknown;
};
declare type TestFixture<R, Args extends KeyValue> = (args: Args, use: (r: R) => Promise<void>) => Promise<void> | void;
declare type TestFixtureValue<R, Args extends KeyValue> = R | TestFixture<R, Args>;
declare type Fixtures<T extends KeyValue, PT extends KeyValue = {}> = {
    [K in keyof PT]?: TestFixtureValue<PT[K], PT & T>;
} & {
    [K in keyof T]: TestFixtureValue<T[K], PT & T>;
};
declare type BaseTest = (name: string, inner: (...args: unknown[]) => Promise<void> | void) => unknown;
declare type Test<Args extends KeyValue, B extends BaseTest> = {
    [key in keyof B]: B[key];
} & {
    (name: string, inner: (args: Args, ...baseArgs: Parameters<Parameters<B>[1]>) => Promise<void> | void): ReturnType<B>;
    extend<T extends KeyValue = {}>(fixtures: Fixtures<T, Args>): Test<Args & T, B>;
};
declare const wrap: <B extends BaseTest = BaseTest>(baseTest: B) => Test<{}, B>;
export default wrap;
