const prepareFixtures = async (base, init) => {
    const extend = {};
    let useResolve;
    let usePromise;
    await new Promise((construct) => {
        usePromise = new Promise((resolve) => { useResolve = resolve; construct(); });
    });
    const finishJobs = [];
    const prepareJobs = Object.entries(init)
        .map(([key, fixtureValue]) => (new Promise((prepareValueResolve) => {
        /**
         * Check if it is callable.
         * Hard to be reliable and fast at the same time.
         */
        if (typeof fixtureValue === 'function') {
            const useValue = async (value) => {
                extend[key] = value;
                prepareValueResolve();
                await usePromise;
            };
            finishJobs.push(
            // Package to promise, another resolve in case of it dont use `useValue`.
            Promise.resolve(fixtureValue(base, useValue))
                .then(prepareValueResolve));
        }
        else {
            extend[key] = fixtureValue;
            prepareValueResolve();
        }
    })));
    await Promise.all(prepareJobs);
    // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
    return [{ ...base, ...extend }, finishJobs, useResolve];
};
const wrapTest = (baseTest, fixturesList) => {
    const proxy = new Proxy(baseTest, {
        apply: (target, thisArg, [name, inner]) => (target.call(thisArg, name, async (...baseTestArgs) => {
            const finishList = [];
            const fixtures = await fixturesList.reduce(async (initializing, init) => {
                const [initialized, finishJobs, finishFunc,] = await prepareFixtures(await initializing, init);
                finishList.push([finishJobs, finishFunc]);
                return initialized;
            }, Promise.resolve({}));
            try {
                await inner.call(thisArg, fixtures, ...baseTestArgs);
            }
            finally {
                await finishList.reduceRight(async (finishing, [finishJobs, finishFunc]) => {
                    await finishing;
                    finishFunc();
                    await Promise.all(finishJobs);
                }, Promise.resolve());
            }
        })),
    });
    return Object.assign(proxy, {
        extend(fixtures) {
            return wrapTest(baseTest, [...fixturesList, fixtures]);
        },
    });
};
const wrap = (baseTest) => wrapTest(baseTest, []);
export default wrap;
